Documentation
¶
Index ¶
Constants ¶
This section is empty.
Variables ¶
var BuiltinModules = map[string]map[string]core.Value{
"math": mathModule,
"os": osModule,
"text": textModule,
"times": timesModule,
"rand": randModule,
"fmt": fmtModule,
"json": jsonModule,
"base64": base64Module,
"hex": hexModule,
}
BuiltinModules are builtin type standard library modules.
var SourceModules = map[string]string{}
SourceModules are source type standard library modules.
Functions ¶
func AllModuleNames ¶
func AllModuleNames() []string
AllModuleNames returns a list of all default module names.
func GetModuleMap ¶
GetModuleMap returns the module map that includes all modules for the given module names.
Types ¶
This section is empty.
Source Files
¶
Click to show internal directories.
Click to hide internal directories.